/*globals CoB:false */
(function($) {
    $.fn.advanceEditorToolbar = function(editor, core) {
        var ToolBarModel = function() {

            this._lastQryHistorySize = 0;
            this._searchResults = [];
            this._highlights = [];
            this._lastQuery = null;
            this._currPos = 0;
            this._totalResults = 0;

        };
        ToolBarModel.prototype = {
            copyPos: function(pos) {
                return {
                    from: function() { return pos.from; },
                    to: function() {return pos.to; }
                };
            },
            hasResults: function() {
                return this._totalResults > 0;
            },
            getSearchResults: function() {
                return this._searchResults;
            },
            getCurrentPos: function() {
                if ((this._totalResults > 0)) {
                    return this._searchResults[this._currPos];
                } else {
                    return {line: 0, ch: 0};
                }
            },
            getCurrentPosIdx: function() {
                return this._currPos;
            },
            reset: function() {
                if (this._mark)this._mark.clear();
                this._lastQryHistorySize = 0;
                this._searchResults = [];
                this._highlights = [];
                this._lastQuery = null;
                this._currPos = 0;
                this._totalResults = 0;
            },
            getLastQuery: function() {
                return this._lastQuery;
            },
            hasContentChangedFromLastQuery: function() {
                return this._lastQryHistorySize !== editor.historySize().undo;
            },
            getTotalResults: function() {
                return this._totalResults;
            },
            searchFor: function(text, highlight, ignoreCase) {
                for (var cursor = editor.getSearchCursor(text, null, ignoreCase); cursor.findNext();) {
                    this._searchResults.push(this.copyPos(cursor.pos));
                    if (highlight) {
                        this.addHighlight(this.copyPos(cursor.pos));
                    }
                }
                this._totalResults = this._searchResults.length;
                this._lastQuery = text;
                this._currPos = 0;
                this._lastQryHistorySize = editor.historySize().undo;
            },
            searchForTextEntries: function(textEntries, ignoreCase){
                var _this = this;

                textEntries.forEach(function(entry){
                    for (var cursor = editor.getSearchCursor(entry, null, ignoreCase); cursor.findNext();) {
                        _this._searchResults.push(_this.copyPos(cursor.pos));
                        _this.addCmdHighlight(_this.copyPos(cursor.pos));

                    }
                    this._currPos = 0;
                });

                this._lastQuery = textEntries.join(",");
                this._currPos = 0;
                this._lastQryHistorySize = editor.historySize().undo;
            },
            nextResult: function(searchWrap) {
                var pos;
                if (this.hasMoreResults(false)) {
                    this._currPos++;
                    pos = this._searchResults[this._currPos];
                } else if (this.isLastResult() && searchWrap) {
                    pos = this.wrapSeach();
                }
                return pos;
            },
            previousResult: function(searchWrap) {
                var pos;
                if (this.hasMoreResults(true)) {
                    this._currPos--;
                    pos = this._searchResults[this._currPos];
                } else if (this.isFirstResult() && searchWrap) {
                    pos = this.wrapSeach(true);
                }
                return pos;
            },
            hasMoreResults: function(reverse) {
                var rst = false;
                if (reverse) {
                    rst = this._currPos > 0;
                } else {
                    rst = this._currPos < (this._totalResults - 1);
                }
                return rst;
            },
            isLastResult: function() {
                return this.hasResults() && (this._currPos === this._totalResults - 1);
            },
            isFirstResult: function() {
                return this.hasResults() && this._currPos === 0;
            },
            getHighlights: function() {
                return this._highlights;
            },
            clearHighlights: function() {
                this._highlights = [];
            },
            addHighlight: function(pos) {
                this._highlights.push(editor.markText(pos.from(), pos.to(), "CodeMirror-matchhighlight"));
            },
            addCmdHighlight: function(pos){
                this._highlights.push(editor.markText(pos.from(), pos.to(), "CodeMirror-unAuthCmdHighlight"));
            },
            addMark: function(pos) {
                if (this._mark)this._mark.clear();
                this._mark = editor.markText(pos.from(), pos.to(), "CodeMirror-searched");
            },
            wrapSeach: function(reverse) {
                if (reverse) {
                    this._currPos = (this._totalResults - 1);
                } else {
                    this._currPos = 0;
                }
                return this._searchResults[this._currPos];
            }
        };

        if (this.length > 1) {
            window.console.log("Can't apply this to an array of elements");
        }


        var _model = new ToolBarModel();
        var _ignoreCase = true,
            _searchWrap = false,
            _highlightAll = false;

        var _view = (function() {
            var _$toolbar,
                _$gotoNextBtn,
                _$gotoPreviousBtn,
                _$ignoreCaseOption,
                _$searchWrapOption,
                _$hightlightAllOption,
                _$findTextInput,
                _$foundCounter,
                _$foundCounterCurr,
                _$foundCounterTotal,
                _$toggleTabs;


            var _init = function() {

                _$toolbar = $('.js-template-toolbar');
                _$gotoNextBtn = $('.js-next-button', _$toolbar);
                _$gotoPreviousBtn = $('.js-previous-button', _$toolbar);
                _$ignoreCaseOption = $('.js-ignore-case', _$toolbar);
                _$searchWrapOption = $('.js-search-wrap', _$toolbar);
                _$hightlightAllOption = $('.js-highlight-all', _$toolbar);
                _$findTextInput = $('.js-find-text-expr', _$toolbar);
                _$foundCounter = $('.js-found-counter', _$toolbar);
                _$foundCounterCurr = $('.js-found-current', _$toolbar);
                _$foundCounterTotal = $('.js-found-total', _$toolbar);
                _$toggleTabs = $('.js-tabs', _$toolbar);

                _$findTextInput.keydown(function(evt) {
                    if (evt.keyCode === 13) {
                        _doSearch(_view.getQueryString(), false);
                    }
                });

                _$gotoNextBtn.click(function() {
                    _nextResult(_searchWrap);
                });

                _$gotoPreviousBtn.click(function() {
                    _previousResult(_searchWrap);
                });

                _$ignoreCaseOption.click(function() {
                    _doToogleIgnoreCase();
                });
                _toogleIgnoreCase(_ignoreCase);

                _$searchWrapOption.click(function() {
                    _doToogleSearchWreap();
                });
                _toogleSearchWrap(_searchWrap);

                _$hightlightAllOption.click(function() {
                    _doToogleHighlightAll();
                });
                _toogleHighlightAll(_highlightAll);

                _$toggleTabs.click(function() {
                    _$toggleTabs.toggleClass('choosen');
                    editor.showTabs(_$toggleTabs.hasClass('choosen'));
                });

                _setNextPreviousBtnStatus({previousBtn: false, nextBtn: false});

                if (_highlightAll) {
                    _setupFindBoxAutoSearch();
                }
            };

            var _reset = function() {
                _$findTextInput.val("");
                _setNextPreviousBtnStatus(false);
            };

            var _setupFindBoxAutoSearch = function() {
                _$findTextInput.keyup(function(event) {
                    event.preventDefault();
                    _doSearch(_getQueryString(), true);
                });
            };

            var _disableAutoSearch = function() {
                _$findTextInput.unbind('keyup');
            };

            var _resetFoundCounter = function() {
                _setFoundCounter(0, 0, false);
            };

            var _setFoundCounter = function(currPos, total, noResults) {
                if (noResults) {
                    _$findTextInput.addClass('findbox-not-found');
                } else {
                    _$findTextInput.removeClass('findbox-not-found');
                }
                _$foundCounterCurr.text(currPos);
                _$foundCounterTotal.text(total);
            };

            var _getQueryString = function() {
                return _$findTextInput.val();
            };

            var _toogleIgnoreCase = function(val) {
                if (val) {
                    _$ignoreCaseOption.addClass('choosen');
                } else {
                    _$ignoreCaseOption.removeClass('choosen');
                }
            };

            var _toogleSearchWrap = function(val) {
                if (val) {
                    _$searchWrapOption.addClass('choosen');
                } else {
                    _$searchWrapOption.removeClass('choosen');
                }
            };

            var _toogleHighlightAll = function(val) {
                if (val) {
                    _$hightlightAllOption.addClass('choosen');
                } else {
                    _$hightlightAllOption.removeClass('choosen');
                }
            };

            var _setNextPreviousBtnStatus = function(btns) {
                if (btns.nextBtn) {
                    _$gotoNextBtn.removeAttr('disabled');
                } else {
                    _$gotoNextBtn.attr('disabled', '');
                }

                if (btns.previousBtn) {
                    _$gotoPreviousBtn.removeAttr('disabled');
                } else {
                    _$gotoPreviousBtn.attr('disabled', '');
                }
            };

            return  {
                init: _init,
                reset: _reset,
                getQueryString: _getQueryString,
                setFoundCounter: _setFoundCounter,
                resetFoundCounter: _resetFoundCounter,
                toogleIgnoreCase: _toogleIgnoreCase,
                toogleHighlightAll: _toogleHighlightAll,
                toogleSearchWrap: _toogleSearchWrap,
                setNextPreviousBtnStatus: _setNextPreviousBtnStatus,
                setupFindBoxAutoSearch: _setupFindBoxAutoSearch,
                disableAutoSearch: _disableAutoSearch,
                unselectShowTabs: function() {
                    _$toggleTabs.removeClass('choosen');
                    editor.showTabs(false);
                }
            };
        })();

        var _doToogleIgnoreCase = function() {

            _ignoreCase = !_ignoreCase;
            _view.toogleIgnoreCase(_ignoreCase);
            _doSearch(_view.getQueryString(), true);
        };

        var _doToogleSearchWreap = function() {
            _searchWrap = !_searchWrap;
            _view.toogleSearchWrap(_searchWrap);

            if (_model.hasResults()) {
                _view.setNextPreviousBtnStatus(
                    {nextBtn: (!_model.isLastResult() || _searchWrap),
                        previousBtn: (!_model.isFirstResult() || _searchWrap)});
            } else {
                _view.setNextPreviousBtnStatus({nextBtn: false, previousBtn: false});
            }
        };

        var _doToogleHighlightAll = function() {

            _highlightAll = !_highlightAll;
            _view.toogleHighlightAll(_highlightAll);

            if (_highlightAll) {
                if ((_view.getQueryString() !== _model.getLastQuery())) {
                    _doSearch(_view.getQueryString(), true);
                } else {
                    _highlightSearched();
                }
                _view.setupFindBoxAutoSearch();
            } else {
                _removeHighlights();
                _view.disableAutoSearch();
            }
        };

        var _nextResult = function(searchWrap) {

            if (_model.hasContentChangedFromLastQuery()) {
                _doSearch(_view.getQueryString(), false);

            } else {
                var pos = _model.nextResult(searchWrap);
                if (pos) {
                    _updateFindResultFeedback();
                    _model.addMark(pos);
                    _select(pos);
                }
            }
        };

        var _previousResult = function(searchWrap) {
            if (_model.hasContentChangedFromLastQuery()) {
                _doSearch(_view.getQueryString(), false);
            } else {
                var pos = _model.previousResult(searchWrap);
                if (pos) {
                    _updateFindResultFeedback();
                    _model.addMark(pos);
                    _select(pos);
                }
            }
        };

        var _updateFindResultFeedback = function() {
            if (_model.hasResults()) {
                _view.setFoundCounter(_model.getCurrentPosIdx() + 1, _model.getTotalResults(), false);
                _view.setNextPreviousBtnStatus(
                    {nextBtn: (!_model.isLastResult() || _searchWrap),
                        previousBtn: (!_model.isFirstResult() || _searchWrap)});
            } else {
                _view.setFoundCounter(0, 0, true);
                _view.setNextPreviousBtnStatus({nextBtn: false, previousBtn: false});
            }
        };

        var _reset = function() {
            _removeHighlights();
            _model.reset();
            _unselect();
            _view.resetFoundCounter();
            _view.unselectShowTabs();
            editor.hideErrorMsg();
        };

        var _highlightSearched = function() {
            var results = _model.getSearchResults();
            for (var i = 0; i < results.length; i++) {
                _model.addHighlight(results[i]);
            }
        };

        var _select = function(pos) {
            editor.setSelection(pos.from(), pos.to());
        };

        var _unselect = function() {
            editor.setSelection({line: 0, ch: 0}, {line: 0, ch: 0});
            editor.setCursor({line: 0, ch: 0});
        };

        var _removeHighlights = function() {
            var marked = _model.getHighlights();
            for (var i = 0; i < marked.length; ++i) {
                marked[i].clear();
            }
            _model.clearHighlights();
        };

        var _doSearch = function(text, optionsChanged) {
            if (text !== _model.getLastQuery() || optionsChanged || _model.hasContentChangedFromLastQuery()) {
                _reset();
                if (!text) return;
                _model.searchFor(text, _highlightAll, _ignoreCase);
                if (_model.hasResults()) {

                    _model.addMark(_model.getCurrentPos());
                    _select(_model.getCurrentPos());

                }
                _updateFindResultFeedback();
            } else {
                _nextResult(_searchWrap);
            }
        };

        var _doMultipleSearch = function(entries, optionsChanged) {
            _reset();

            if (!entries || entries.length === 0) return;

            _model.searchForTextEntries(entries, _ignoreCase);

            if (_model.hasResults()) {

                _select(_model.getCurrentPos());

            }
        };

        _view.init();

        return {
            searchForMultipleEntries: _doMultipleSearch,
            reset: function() {
                _reset();
                _view.reset();
            }
        };
    };
}(jQuery));
